import { createDisposable } from "@reactive-js/disposable";
import { createPrioritySchedulerResource, createSchedulerWithPriority as createSchedulerWithPriorityImpl, AbstractScheduler, } from "@reactive-js/schedulers";
const performance = window.performance;
const Date = window.Date;
const setTimeout = window.setTimeout;
const clearTimeout = window.clearTimeout;
const now = typeof performance === "object" && typeof performance.now === "function"
    ? () => performance.now()
    : () => Date.now();
const yieldInterval = 5;
const maxYieldInterval = 300;
const shouldCallbackYield = navigator !== undefined &&
    navigator.scheduling !== undefined &&
    navigator.scheduling.isInputPending !== undefined
    ? (startTime) => {
        const currentTime = now();
        const deadline = startTime + yieldInterval;
        const maxDeadline = startTime + maxYieldInterval;
        const inputPending = navigator.scheduling.isInputPending();
        return ((currentTime >= deadline && inputPending) ||
            currentTime >= maxDeadline);
    }
    : (startTime) => now() >= startTime + yieldInterval;
let channel = undefined;
const scheduleImmediate = (callback) => {
    const disposable = createDisposable();
    channel = channel || new MessageChannel();
    channel.port1.onmessage = () => {
        if (!disposable.isDisposed) {
            callback();
            disposable.dispose();
        }
    };
    channel.port2.postMessage(null);
    return disposable;
};
const callCallbackAndDispose = (callback, disposable) => {
    callback();
    disposable.dispose();
};
const scheduleDelayed = (callback, delay = 0) => {
    const disposable = createDisposable(() => clearTimeout(timeout));
    const timeout = setTimeout(callCallbackAndDispose, delay, callback, disposable);
    return disposable;
};
class WebScheduler extends AbstractScheduler {
    shouldCallbackYield(startTime) {
        return shouldCallbackYield(startTime);
    }
    scheduleCallback(callback, delay = 0) {
        return delay >= 4
            ? scheduleDelayed(callback, delay)
            : scheduleImmediate(callback);
    }
    get now() {
        return now();
    }
}
let schedulerHost = undefined;
let priorityScheduler = undefined;
export const createSchedulerWithPriority = (priority) => {
    schedulerHost = schedulerHost || new WebScheduler();
    priorityScheduler =
        priorityScheduler || createPrioritySchedulerResource(schedulerHost);
    return createSchedulerWithPriorityImpl(priorityScheduler, priority);
};
//# sourceMappingURL=scheduler.js.map